package org.maltparser.core.config;

import java.io.BufferedWriter;
import java.io.File;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

import org.apache.log4j.Logger;
import org.maltparser.core.exception.MaltChainedException;

/**
 *  An interface for creating different types of configurations. A MaltEngine invokes the "front-end" 
 *  methods initialize(), process() and finalize() and there are several methods for "back-end" classes to
 *  get information from the configuration.
 *
 * @author Johan Hall
 * @since 1.0
**/
public interface Configuration {
	public void initialize(Integer containerIndex) throws MaltChainedException;
	public void process() throws MaltChainedException;
	public void finalize() throws MaltChainedException; 
	
	public Logger getConfigLogger(); 
	public String getName();
	public int getMode();
	public Object getOptionValue(String optiongroup, String optionname) throws MaltChainedException;
	public OutputStreamWriter getOutputStreamWriter(String suffix) throws MaltChainedException;
	public InputStreamReader getInputStreamReader(String suffix) throws MaltChainedException;
	public File getFile(String suffix) throws MaltChainedException;
	public BufferedWriter getInfoFileWriter() throws MaltChainedException;
}
