package org.maltparser.core.graph;

import java.util.HashMap;
import java.util.SortedMap;

import org.maltparser.core.dataformat.ColumnDescription;
import org.maltparser.core.exception.MaltChainedException;
import org.maltparser.core.graph.arc.ArcLabels;
import org.maltparser.core.symbol.SymbolTable;

/**
 *  
 *
 * @author Johan Hall
 * @since 1.0
**/
public class RootLabels {
	public final static String DEFAULT_ROOTSYMBOL = "ROOT";
	private HashMap<SymbolTable,Integer> rootLabelCodes;

	
	public RootLabels(String rootLabelOption, SortedMap<String,ColumnDescription> outputColumns) throws MaltChainedException {
		rootLabelCodes = new HashMap<SymbolTable,Integer>();
		if (rootLabelOption == null || rootLabelOption.trim().length() == 0) {
			for (ColumnDescription column : outputColumns.values()) {
				rootLabelCodes.put(column.getSymbolTable(), column.getSymbolTable().addSymbol(RootLabels.DEFAULT_ROOTSYMBOL));
			}
		} else if (rootLabelOption.trim().indexOf('|') == -1) {
			int index = rootLabelOption.trim().indexOf('=');
			if (index == -1) {
				for (ColumnDescription column : outputColumns.values()) {
					rootLabelCodes.put(column.getSymbolTable(), column.getSymbolTable().addSymbol(rootLabelOption.trim()));
				}
			} else {
				String outputColumnName = rootLabelOption.trim().substring(0, index);
				ColumnDescription column = outputColumns.get(outputColumnName);
				if (column == null) {
					throw new GraphException("The output column '"+ outputColumnName+"' cannot be found when defining the root symbol. ");
				} else {
					rootLabelCodes.put(column.getSymbolTable(), column.getSymbolTable().addSymbol(rootLabelOption.trim().substring(index+1)));
					if (outputColumns.size() > 1) {
						for (ColumnDescription col : outputColumns.values()) {
							if (!col.getName().equals(outputColumnName)) {
								rootLabelCodes.put(column.getSymbolTable(), column.getSymbolTable().addSymbol(RootLabels.DEFAULT_ROOTSYMBOL));					
							}
						}
					}
				}
			}
		} else {
			String[] items = rootLabelOption.trim().split("|");
			for (int i=0; i<items.length; i++) {
				int index = items[i].trim().indexOf('=');
				if (index == -1) {
					throw new GraphException("The root symbol is undefinied. ");
				} else {
					String outputColumnName = items[i].trim().substring(0, index);
					ColumnDescription column = outputColumns.get(outputColumnName);
					if (column == null) {
						throw new GraphException("The output column '"+ outputColumnName+"' cannot be found when defining the root symbol. ");
					} else {
						rootLabelCodes.put(column.getSymbolTable(), column.getSymbolTable().addSymbol(items[i].trim().substring(index+1)));
					}
				}
			}
			for (ColumnDescription column : outputColumns.values()) {
				if (!rootLabelCodes.containsKey(column.getSymbolTable())) {
					rootLabelCodes.put(column.getSymbolTable(), column.getSymbolTable().addSymbol(RootLabels.DEFAULT_ROOTSYMBOL));					
				}
			}
		}
	}
	
	public Integer getRootLabelCode(SymbolTable table) {
		return rootLabelCodes.get(table);
	}
	
	public String getRootLabel(SymbolTable table) throws MaltChainedException {
		return table.getSymbolCodeToString(rootLabelCodes.get(table));
	}
	
	
	public boolean checkRootLabelCodes(ArcLabels rlc) {
		if (rlc == null && rootLabelCodes == null) {
			return true; // or false ?
		} else if ((rlc == null && rootLabelCodes != null) || (rlc != null && rootLabelCodes == null)) {
			return false;
		} else if (rlc.size() != rootLabelCodes.size()) {
			return false;
		} else {
			for (SymbolTable table : rootLabelCodes.keySet()) {
				if (!rootLabelCodes.get(table).equals(rlc.get(table))) {
					return false;
				}
			}
			return true;
		}
	}
	
	/* public String toString() {
		StringBuilder sb = new StringBuilder();
		for (SymbolTable table : rootLabelCodes.keySet()) {
			sb.append(table.getName()+"="+ table.getSymbolCodeToString(rootLabelCodes.get(table))+ "("+rootLabelCodes.get(table).toString()+")\n");
		}
		return sb.toString();
	}*/
}
