package org.maltparser.core.graph.arc;

import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

import org.maltparser.core.graph.Node;
import org.maltparser.core.symbol.SymbolTable;
/**
 *  
 *
 * @author Johan Hall
 * @since 1.0
**/
public class MultipleHeadArcs implements HeadArcs {
	protected SortedMap<Node, ArcLabels> labelCodes;
	protected SortedSet<Node> heads;
	
	public MultipleHeadArcs() {
		labelCodes = new TreeMap<Node, ArcLabels>();
		heads = new TreeSet<Node>();
	}
	
	public void clear() {
		labelCodes.clear();
		heads.clear();
	}

	public boolean isSingleHeaded() {
		return !(heads.size() > 1);
	}
	
	public boolean hasHead() {
		return !heads.isEmpty();
	}
	public Node getHead() {
		return heads.first();
	}
	public Node getHead(int index) {
		for (Node head : heads) {
			if (head.getIndex().intValue() == index) {
				return head;
			}
		}
		return null;
	}
	public SortedSet<Node> getHeads() {
		return heads;
	}
	public Integer getLabelCode(int headindex, SymbolTable table) {
		if (labelCodes.containsKey(getHead(headindex))) {
			return labelCodes.get(getHead(headindex)).get(table);
		}
		return null;
	}
	public ArcLabels getArcLabels(int headindex) {
		return labelCodes.get(getHead(headindex));
	}
	public ArcLabels getFirstHeadArcLabels() {
		if (labelCodes.firstKey() != null) {
			return labelCodes.get(labelCodes.firstKey());
		}
		return null;
	}
	public Integer getFirstHeadLabelCode(SymbolTable table) {
		if (labelCodes.firstKey() != null) {
			return labelCodes.get(labelCodes.firstKey()).get(table);
		}
		return null;
	}
	public Integer getLabelCode(Node head, SymbolTable table) {
		if (head != null && labelCodes.containsKey(head)) {
			return labelCodes.get(head).get(table);
		}
		return null;
	}
	public boolean hasLabelCodes() {
		if (labelCodes.isEmpty()) {
			return false;
		} else {
			if (labelCodes.firstKey() != null) {
				return !labelCodes.get(labelCodes.firstKey()).isEmpty();
			}
		}
		return false;
	}
	public void addHead(Node head, ArcLabels arcLabelCodes) {
		heads.add(head);
		if (arcLabelCodes != null) {
			labelCodes.put(head, arcLabelCodes); // maybe have to be cloned
		}
	}
}
