package org.maltparser.parser.guide.feature;

import org.maltparser.core.dataformat.ColumnDescription;
import org.maltparser.core.exception.MaltChainedException;
import org.maltparser.core.symbol.nullvalue.NullValues.NullValueId;
import org.maltparser.parser.algorithm.ParsingAlgorithm;


/**


@author Johan Hall
@since 1.0
 */
public class OutputColumnFeature extends ColumnFeature {

	public OutputColumnFeature(ParsingAlgorithm parsingAlgorithm, String spec) throws MaltChainedException {
		super();
		super.initialize(parsingAlgorithm);
		spec = spec.trim();
		int index = spec.indexOf(',');
		if (index == -1) {
			throw new FeatureException("The column name cannot be found at feature line '"+spec+"'. ");
		}
		ColumnDescription column = parsingAlgorithm.getDependencyGraph().getDataFormatInstance().getColumnDescriptionByName(spec.substring(0, index));
		if (column == null) {
			throw new FeatureException("The column '"+spec.substring(0, index)+"' cannot be found. ");
		}
		if (column.getCategory() != ColumnDescription.OUTPUT) {
			throw new FeatureException("The column '"+column.getName()+"' is not an output column at line '"+spec+"'. ");
		}
		super.setColumn(column);
		if (!init(spec.substring(index+1))) { 
			throw new FeatureException("The output column feature could not be initialized at line '"+spec+"'. ");
		}
	}
	
	public boolean init(String spec) throws MaltChainedException {
		return super.init(spec);
	}

	public void update()  throws MaltChainedException {
		featureValues.clear();
		featureKnown.clear();
		currentUpdateNode = getNode();
		if (currentUpdateNode != null) {
			if (!currentUpdateNode.isRoot()) {
				if (currentUpdateNode.hasHead()) {
					featureValues.put(currentUpdateNode.getLabelCode(column.getSymbolTable()), column.getSymbolTable().getSymbolCodeToString(currentUpdateNode.getLabelCode(column.getSymbolTable())));
					featureKnown.put(currentUpdateNode.getLabelCode(column.getSymbolTable()), column.getSymbolTable().getKnown(currentUpdateNode.getLabelCode(column.getSymbolTable())));
					nullValue = false;
				} else {
					featureValues.put(column.getSymbolTable().getNullValueCode(NullValueId.NO_VALUE), column.getSymbolTable().getNullValueSymbol(NullValueId.NO_VALUE));
					featureKnown.put(column.getSymbolTable().getNullValueCode(NullValueId.NO_VALUE), true);
					nullValue = true;
				}
			} else {
				featureValues.put(column.getSymbolTable().getNullValueCode(NullValueId.ROOT_NODE), column.getSymbolTable().getNullValueSymbol(NullValueId.ROOT_NODE));
				featureKnown.put(column.getSymbolTable().getNullValueCode(NullValueId.ROOT_NODE), true);
				nullValue = true;
			}
		} else {
			featureValues.put(column.getSymbolTable().getNullValueCode(NullValueId.NO_NODE), column.getSymbolTable().getNullValueSymbol(NullValueId.NO_NODE));
			featureKnown.put(column.getSymbolTable().getNullValueCode(NullValueId.NO_NODE), true);
			nullValue = true;
		}
	}
	
	public boolean equals(Object obj) {
		if (!(obj instanceof OutputColumnFeature)) {
			return false;
		}
		if (!obj.toString().equals(this.toString())) {
			return false;
		}
		return true;
	}
	
	public String toString() {
		return "OutputColumn(" + super.toString() + ")";
	}
}
