package org.maltparser.parser.guide.feature;

import java.util.Stack;

import org.maltparser.core.exception.MaltChainedException;
import org.maltparser.core.graph.Node;

/**


@author Johan Hall
@since 1.0
 */
public abstract class TokenFeature extends AtomicFeature {
	private Stack<String> operations;
	private int indexDataStructure;
	private Node staticNode; // used by createOperationStack to ask the node if a operation is legal or not
	public TokenFeature() throws MaltChainedException {
		super();	
		operations = new Stack<String>();
		staticNode = new Node(0,true,true);
	}
	
	protected boolean createOperationStack(String op) throws MaltChainedException {
		int indexp = op.indexOf('(');
		int indexb = op.indexOf('[');
		
		if (indexb == -1) {
			return false;
		}
		if (indexp != -1 && indexp >= indexb) {
			return false;
		}
		if (indexp != -1) {
			if (op.charAt(op.length()-1) == ')' && staticNode.isOperation(op.substring(0,indexp))) {
				operations.push(op.substring(0,indexp));
				return createOperationStack(op.substring(indexp+1,op.length()-1));
			} else {
				return false;
			}
		} else {
			if (op.charAt(op.length()-1) == ']' && parsingAlgorithm.isOperation(op.substring(0,indexb))) {
				operations.push(op.substring(0,indexb));
				indexDataStructure = Integer.parseInt(op.substring(indexb+1,op.length()-1));
			} else {
				return false;
			}			
		}
		return true;
	}
	
	protected Node getNode() throws MaltChainedException {
		Node node = parsingAlgorithm.getNode(operations.get(operations.size()-1), indexDataStructure);
		
		for (int i = operations.size()-2; i >= 0; i--) {
			if (node == null) {
				return null;
			}
			node = node.getNode(operations.get(i), 0);
		}
		return node;
	}
	
	public void replaceStackItem(String replaceOperation, String newOperation) {
		for (int i = 0; i < operations.size(); i++) {
			if (operations.get(i).equals(replaceOperation)) {
				operations.set(i, newOperation);
			}
		}
	}
	
	public void setCardinality(int cardinality) {
		super.setCardinality(cardinality);
	}
	
	
	public boolean equals(Object obj) {
		if (!(obj instanceof TokenFeature)) {
			return false;
		}
		if (!obj.toString().equals(this.toString())) {
			return false;
		}
		return true;
	}
	
	public String toString() {
		return super.toString();
	}
}
